from tf_geometric.layers.conv.gcn import GCN
from tf_geometric.layers.conv.gat import GAT
from tf_geometric.layers.kernel.map_reduce import MapReduceGNN
from tf_geometric.layers.pool.mean_pool import MeanPool
from tf_geometric.layers.conv.gin import GIN
